function [ x_new, indx_new, k_new, nstack_new, ncan_new ] = i4vec_backtrack ( ...
  n, maxstack, stack, x, indx, k, nstack, ncan )

%% I4VEC_BACKTRACK supervises a backtrack search for an integer vector.
%
%  Discussion:
%
%    The routine tries to construct a integer vector one index at a time,
%    using possible candidates as supplied by the user.
%
%    At any time, the partially constructed vector may be discovered to be
%    unsatisfactory, but the routine records information about where the
%    last arbitrary choice was made, so that the search can be
%    carried out efficiently, rather than starting out all over again.
%
%    First, call the routine with INDX = 0 so it can initialize itself.
%
%    Now, on each return from the routine, if INDX is:
%      1, you've just been handed a complete candidate vector;
%         Admire it, analyze it, do what you like.
%      2, please determine suitable candidates for position X(K).
%         Return the number of candidates in NCAN(K), adding each
%         candidate to the end of STACK, and increasing NSTACK.
%      3, you're done.  Stop calling the routine;
%
%  Modified:
%
%    12 July 2004
%
%  Reference:
%
%    A Nijenhuis and H Wilf,
%    Combinatorial Algorithms,
%    Academic Press, 1978, second edition,
%    ISBN 0-12-519260-6.
%
%  Parameters:
%
%    Input, integer N, the number of positions to be filled in the vector.
%
%    Input, integer MAXSTACK, the maximum length of the stack.
%
%    Input, integer STACK(MAXSTACK), a list of all current candidates for
%    all positions 1 through K.
%
%    Input, integer X(N), the partially filled in candidate vector.
%
%    Input, integer INDX, a communication flag.
%    * 0, to begin a backtracking search.
%    * 2, the requested candidates for position K have been added to 
%      STACK, and NCAN(K) was updated.
%
%    Input, integer K, the index in X that we are trying to fill.
%
%    Input, integer NSTACK, the current length of the stack.
%
%    Input, integer NCAN(N), lists the current number of candidates for
%    all positions 1 through K.
%
%    Output, integer X_NEW(N), the partially filled in candidate vector.
%
%    Output, integer INDX_NEW, a communication flag.
%    * 1, a complete output vector has been determined and returned in X(1:N);
%    * 2, candidates are needed for position X(K);
%    * 3, no more possible vectors exist.
%
%    Output, integer K_NEW, the index in X that we are trying to fill.
%
%    Output, integer NSTACK_NEW, the current length of the stack.
%
%    Output, integer NCAN_NEW(N), lists the current number of candidates for
%    all positions 1 through K.
%
  x_new(1:n) = x(1:n);
  indx_new = indx;
  k_new = k;
  nstack_new = nstack;
  ncan_new(1:n) = ncan(1:n);
%
%  If this is the first call, request a candidate for position 1.
%
  if ( indx_new == 0 )
    k_new = 1;
    nstack_new = 0;
    indx_new = 2;
    return
  end
%
%  Examine the stack.
%
  while ( 1 )
%
%  If there are candidates for position K, take the first available
%  one off the stack, and increment K.
%
%  This may cause K to reach the desired value of N, in which case
%  we need to signal the user that a complete set of candidates
%  is being returned.
%
    if ( 0 < ncan(k_new) )
      x_new(k_new) = stack(nstack_new);
      nstack_new = nstack_new - 1;

      ncan_new(k_new) = ncan_new(k_new) - 1;

      if ( k_new ~= n )
        k_new = k_new + 1;
        indx_new = 2;
      else
        indx_new = 1;
      end

      break
%
%  If there are no candidates for position K, then decrement K.
%  If K is still positive, repeat the examination of the stack.
%
    else

      k_new = k_new - 1;

      if ( k_new <= 0 )
        indx_new = 3;
        break
      end

    end

  end
